import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.image.*;

public class PictureViewer implements MouseMotionListener, ActionListener, MouseListener
{
	//Main gui variables
	private JFrame pictureFrame;
	private JPanel picturePanel;
	private JPanel panelLayout;
	private JScrollPane scrollPicture;
	
	//information bar variables
	private JLabel xValue;
	private JLabel yValue;
	private JLabel rValue;
	private JLabel gValue;
	private JLabel bValue;
	private JPanel colorLabel;
	
	//the picture and zoom picture
	private JavaPicture picture;
	private JavaPicture zoomPicture;
	
	//the zoom factor
	private double zoomFactor;
	
	//the menu components
	private JMenuBar menuBar;
	private JMenu zoomMenu;
	private JMenuItem twentyFive;
	private JMenuItem fifty;
	private JMenuItem seventyFive;
	private JMenuItem hundred;
	private JMenuItem hundredFifty;
	private JMenuItem twoHundred;
	private JMenuItem fiveHundred;
	
	/**
     * Public constructor 
     * 
     * @param JavaPicture picture
     */
	public PictureViewer(JavaPicture picture)
	{
		this.picture=picture;
		picturePanel = picture.getCanvas();
		picturePanel.addMouseMotionListener(this);
		picturePanel.addMouseListener(this);
		zoomPicture=picture;
		zoomFactor=1;
		createWindow();
		if(picture.getWidth()==-1)
		{
			pictureFrame.dispose();
		}
	}

     public void setTitle(String title)
     {
	pictureFrame.setTitle(title);
     }
	
	/**
     * Makes the frame and the each part of the GUI
     * and displays it
     *
     */
	private void createWindow()
	{
		int width;
		int height;
		pictureFrame = new JFrame(picture.getFileName());
		
		//create menu
		menuBar = new JMenuBar();
		zoomMenu = new JMenu("Zoom");
		twentyFive = new JMenuItem("25%");
		fifty = new JMenuItem("50%");
		seventyFive = new JMenuItem("75%");
		hundred = new JMenuItem("100%");
		hundred.setEnabled(false);
		hundredFifty = new JMenuItem("150%");
		twoHundred = new JMenuItem("200%");
		fiveHundred = new JMenuItem("500%");
		
		twentyFive.addActionListener(this);
		fifty.addActionListener(this);
		seventyFive.addActionListener(this);
		hundred.addActionListener(this);
		hundredFifty.addActionListener(this);
		twoHundred.addActionListener(this);
		fiveHundred.addActionListener(this);
		
		zoomMenu.add(twentyFive);
		zoomMenu.add(fifty);
		zoomMenu.add(seventyFive);
		zoomMenu.add(hundred);
		zoomMenu.add(hundredFifty);
		zoomMenu.add(twoHundred);
		zoomMenu.add(fiveHundred);
		
		menuBar.add(zoomMenu);
		
		pictureFrame.setJMenuBar(menuBar);
		
		//sets the frame size based on picture
		setFrameSize();
		pictureFrame.setResizable(true);
		pictureFrame.getContentPane().setLayout(new BorderLayout());
		pictureFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		
		//creates the information panel
		createInfoBar();
		
		//creates the scrollpane for the picture
		scrollPicture = new JScrollPane();
		scrollPicture.setViewportView(picturePanel);
		pictureFrame.getContentPane().add(scrollPicture, BorderLayout.CENTER);
		
		pictureFrame.setVisible(true);
	}
	
	/**
     * Creates the North JPanel with all the information
     *
     */
	private void createInfoBar()
	{
		Font largerFont;
		panelLayout= new JPanel();
		panelLayout.setPreferredSize(new Dimension(350,30));
	
		xValue = new JLabel("X: N/A");
		yValue = new JLabel("Y: N/A");
		rValue = new JLabel("R: N/A");
		gValue = new JLabel("G: N/A");
		bValue = new JLabel("B: N/A");
		colorLabel = new JPanel();
		
		panelLayout.setBackground(Color.black);
		
		colorLabel.setBackground(Color.black);
		xValue.setForeground(Color.white);
		yValue.setForeground(Color.white);
		rValue.setForeground(Color.white);
		gValue.setForeground(Color.white);
		bValue.setForeground(Color.white);
		
		largerFont = new Font(xValue.getFont().getName(),xValue.getFont().getStyle(),14);
		xValue.setFont(largerFont);
		yValue.setFont(largerFont);
		rValue.setFont(largerFont);
		gValue.setFont(largerFont);
		bValue.setFont(largerFont);
		
		xValue.setPreferredSize(new Dimension(50,25));
		yValue.setPreferredSize(new Dimension(50,25));
		rValue.setPreferredSize(new Dimension(50,25));
		gValue.setPreferredSize(new Dimension(50,25));
		bValue.setPreferredSize(new Dimension(50,25));
		colorLabel.setPreferredSize(new Dimension(25,25));
		
		panelLayout.add(xValue);
		panelLayout.add(yValue);
		panelLayout.add(rValue);
		panelLayout.add(gValue);
		panelLayout.add(bValue);
		panelLayout.add(colorLabel);
		
		pictureFrame.getContentPane().add(panelLayout, BorderLayout.NORTH);
		
	}
	
	/**
     * Create the frame size based on the picture
     *
     */
	private void setFrameSize()
	{
		int width;
		int height;
		
		if(zoomPicture.getWidth()<350)
		{
			width=350;
		}
		else if(zoomPicture.getWidth()>550)
		{
			width=550;
		}
		else
		{
			width=zoomPicture.getWidth()+12;
		}
		
		if(zoomPicture.getHeight()<225)
		{
			height=271;
		}
		else if(zoomPicture.getHeight()>550)
		{
			height=550;
		}
		else
		{
			height=zoomPicture.getHeight()+85;
		}
		pictureFrame.setSize(width,height);
	}
	
	/**
     * Zooms in the on picture by scaling the image.  
     * It is extremely memory intensive.
     *
     * @param double factor
     */
	private void zoom(double factor)
	{
		try
		{
			if(!(zoomPicture.equals(picture)))
			{
				zoomPicture.dispose();
			}
			zoomFactor=factor;
			zoomPicture = new JavaPicture();
			int width=(int)(picture.getWidth()*zoomFactor);
			int height=(int)(picture.getHeight()*zoomFactor);
			zoomPicture.zoomLoadImage(picture.getBufferedImage().getScaledInstance(width,
								height,picture.getBufferedImage().SCALE_FAST),width,height);
			picturePanel = zoomPicture.getCanvas();
			scrollPicture.setViewportView(picturePanel);
			picturePanel.addMouseMotionListener(this);
			pictureFrame.repaint();
		}
		catch(NullPointerException e)
		{
		}
	}
	

	//****************************************//
	//               Event Listeners          //
	//****************************************//
	
	/**
     * Controls the updating of the information values.
     *
     * @param MouseEvent e
     */
	public void mouseDragged(MouseEvent e)
	{
		try
		{
			//information for x y and rgb 
			//adjusts by zoomfactor to make pixels accurate
			//it reflects the pixels of the actual image
			//not the scaled image
			JavaPixel pixel = new JavaPixel(zoomPicture, e.getX(), e.getY());
			xValue.setText("X: " + (int)(e.getX()/zoomFactor));
			yValue.setText("Y: " + (int)(e.getY()/zoomFactor));
			rValue.setText("R: " + pixel.getRed());
			gValue.setText("G: " + pixel.getGreen());
			bValue.setText("B: " + pixel.getBlue());
			colorLabel.setBackground(new Color(pixel.getRed(), pixel.getGreen(), pixel.getBlue()));
			
			
		}
		catch(ArrayIndexOutOfBoundsException exception)
		{
			xValue.setText("X: N/A");
			yValue.setText("Y: N/A");
			rValue.setText("R: N/A");
			gValue.setText("G: N/A");
			bValue.setText("B: N/A");
			colorLabel.setBackground(Color.black);

		}
		catch(NullPointerException exception)
		{
		}
		
		
	}
	
	public void mouseMoved(MouseEvent e)
	{
	}
	
	public void mouseClicked(MouseEvent e)
	{
	}
	
	public void mousePressed(MouseEvent e)
	{
		try
		{
			//information for x y and rgb 
			//adjusts by zoomfactor to make pixels accurate
			//it reflects the pixels of the actual image
			//not the scaled image
			JavaPixel pixel = new JavaPixel(zoomPicture, e.getX(), e.getY());
			xValue.setText("X: " + (int)(e.getX()/zoomFactor));
			yValue.setText("Y: " + (int)(e.getY()/zoomFactor));
			rValue.setText("R: " + pixel.getRed());
			gValue.setText("G: " + pixel.getGreen());
			bValue.setText("B: " + pixel.getBlue());
			colorLabel.setBackground(new Color(pixel.getRed(), pixel.getGreen(), pixel.getBlue()));
			
			
		}
		catch(ArrayIndexOutOfBoundsException exception)
		{
			xValue.setText("X: N/A");
			yValue.setText("Y: N/A");
			rValue.setText("R: N/A");
			gValue.setText("G: N/A");
			bValue.setText("B: N/A");
			colorLabel.setBackground(Color.black);

		}
		catch(NullPointerException exception)
		{
		}
	}
	
	public void mouseReleased(MouseEvent e)
	{
	}
	
	public void mouseEntered(MouseEvent e)
	{
	}
	
	public void mouseExited(MouseEvent e)
	{
	}
	/**
     * Controls the zoom menu bar
     *
     * @param ActionEvent a
     */
	public void actionPerformed(ActionEvent a)
	{
		if(a.getActionCommand().equals("25%"))
		{
			zoom(.25);
			twentyFive.setEnabled(false);
			fifty.setEnabled(true);
			seventyFive.setEnabled(true);
			hundred.setEnabled(true);
			hundredFifty.setEnabled(true);
			twoHundred.setEnabled(true);
			fiveHundred.setEnabled(true);
		}
		
		if(a.getActionCommand().equals("50%"))
		{
			zoom(.50);
			twentyFive.setEnabled(true);
			fifty.setEnabled(false);
			seventyFive.setEnabled(true);
			hundred.setEnabled(true);
			hundredFifty.setEnabled(true);
			twoHundred.setEnabled(true);
			fiveHundred.setEnabled(true);
		}
		
		if(a.getActionCommand().equals("75%"))
		{
			zoom(.75);
			twentyFive.setEnabled(true);
			fifty.setEnabled(true);
			seventyFive.setEnabled(false);
			hundred.setEnabled(true);
			hundredFifty.setEnabled(true);
			twoHundred.setEnabled(true);
			fiveHundred.setEnabled(true);
		}
		
		if(a.getActionCommand().equals("100%"))
		{
			zoom(1.0);
			twentyFive.setEnabled(true);
			fifty.setEnabled(true);
			seventyFive.setEnabled(true);
			hundred.setEnabled(false);
			hundredFifty.setEnabled(true);
			twoHundred.setEnabled(true);
			fiveHundred.setEnabled(true);
		}
		
		if(a.getActionCommand().equals("150%"))
		{
			zoom(1.5);
			twentyFive.setEnabled(true);
			fifty.setEnabled(true);
			seventyFive.setEnabled(true);
			hundred.setEnabled(true);
			hundredFifty.setEnabled(false);
			twoHundred.setEnabled(true);
			fiveHundred.setEnabled(true);
		}
		
		if(a.getActionCommand().equals("200%"))
		{
			zoom(2.0);
			twentyFive.setEnabled(true);
			fifty.setEnabled(true);
			seventyFive.setEnabled(true);
			hundred.setEnabled(true);
			hundredFifty.setEnabled(true);
			twoHundred.setEnabled(false);
			fiveHundred.setEnabled(true);
		}
		
		if(a.getActionCommand().equals("500%"))
		{
			zoom(5.0);
			twentyFive.setEnabled(true);
			fifty.setEnabled(true);
			seventyFive.setEnabled(true);
			hundred.setEnabled(true);
			hundredFifty.setEnabled(true);
			twoHundred.setEnabled(true);
			fiveHundred.setEnabled(false);
		}
	}
	//****************************************//
	//              Test Main                 //
	//****************************************//
	
	/**
     * Test Main
     */
	public static void main( String args[])
    {
		JavaPicture p = new JavaPicture();
		p.loadImage("\\School Work\\Mediacomp\\temp.jpg");
		
		PictureViewer test;
		test= new PictureViewer(p);
		//test.zoom(2);
    }
}