import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class PictureViewer implements MouseMotionListener
{
	JFrame pictureFrame;
	JPanel picturePanel;
	JScrollPane scrollPicture;
	JLabel xValue;
	JLabel yValue;
	JLabel rValue;
	JLabel gValue;
	JLabel bValue;
	JPanel zoomPanel;
	JavaPicture picture;
	JavaPicture zoomPic;
	
	/**
     * Public constructor 
     * 
     * @param JavaPicture picture
     */
	public PictureViewer(JavaPicture picture)
	{
		this.picture=picture;
		picturePanel=picture.getCanvas();
		picturePanel.addMouseMotionListener(this);
		
		zoomPic = new JavaPicture();
		zoomPic.createNewImage(71,101);
		zoomPanel=zoomPic.getCanvas();
		
		createWindow();
	

	}
	
	/**
     * Makes the frame and the each part of the GUI
     * and displays it
     *
     */
	private void createWindow()
	{
		pictureFrame = new JFrame(picture.getFileName());
		pictureFrame.setSize(450,550);
		pictureFrame.setResizable(false);
		pictureFrame.getContentPane().setLayout(new BorderLayout());
		pictureFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		createInfoBar();
		
		
		scrollPicture = new JScrollPane(picturePanel);
		pictureFrame.getContentPane().add(scrollPicture, BorderLayout.CENTER);
		pictureFrame.setVisible(true);
	}
	
	/**
     * Creates the North JPanel with all the information
     *
     */
	private void createInfoBar()
	{
		Font largerFont;
		JPanel panelLayout= new JPanel();
		panelLayout.setPreferredSize(new Dimension(450,70));
	
		xValue = new JLabel("X: N/A");
		yValue = new JLabel("Y: N/A");
		rValue = new JLabel("R: N/A");
		gValue = new JLabel("G: N/A");
		bValue = new JLabel("B: N/A");
		
		panelLayout.setBackground(Color.black);
		xValue.setForeground(Color.white);
		yValue.setForeground(Color.white);
		rValue.setForeground(Color.red);
		gValue.setForeground(Color.green);
		bValue.setForeground(Color.blue);
		
		largerFont = new Font(xValue.getFont().getName(),xValue.getFont().getStyle(),14);
		xValue.setFont(largerFont);
		yValue.setFont(largerFont);
		rValue.setFont(largerFont);
		gValue.setFont(largerFont);
		bValue.setFont(largerFont);
		
		xValue.setPreferredSize(new Dimension(60,70));
		yValue.setPreferredSize(new Dimension(60,70));
		rValue.setPreferredSize(new Dimension(60,70));
		gValue.setPreferredSize(new Dimension(60,70));
		bValue.setPreferredSize(new Dimension(60,70));
		
		panelLayout.add(xValue);
		panelLayout.add(yValue);
		panelLayout.add(rValue);
		panelLayout.add(gValue);
		panelLayout.add(bValue);
		panelLayout.add(zoomPanel);
		
		pictureFrame.getContentPane().add(panelLayout, BorderLayout.NORTH);
		
	}
	
	/**
     * Enlarges each pixel to give the affect of zoom
     *
     * @param int x
     * @param int y
     * @param int rgb
     */
	
	public void createZoom(int x, int y, int rgb)
	{
		int i;
		int j;
		for(i=x-5;i<=x+5;i++)
		{
			for(j=y-5;j<=y+5;j++)
			{
				zoomPic.setBasicPixel(i,j,rgb);
			}
		}
	}
	
	
	//****************************************//
	//               Event Listeners          //
	//****************************************//
	
	public void mouseMoved(MouseEvent e)
	{
		try
		{
			//information for x y and rgb 
			JavaPixel pixel = new JavaPixel(picture, e.getX(), e.getY());
			xValue.setText("X: " + e.getX());
			yValue.setText("Y: " + e.getY());
			rValue.setText("R: " + pixel.getRed());
			gValue.setText("G: " + pixel.getGreen());
			bValue.setText("B: " + pixel.getBlue());
			
			//creates zoom window
			
			if(e.getX()-2>=0 && e.getY()-2>=0 && 
				e.getX()+2<=picture.getSize().getWidth() && 
				e.getY()+2<=picture.getSize().getHeight())
			{
				//obtains rgb values for pixels in the pictures
				int center = picture.getBasicPixel(e.getX(), e.getY());
			
				int down1 = picture.getBasicPixel(e.getX(), e.getY()+1);
				int down2 = picture.getBasicPixel(e.getX(), e.getY()+2);
	
				int up1 = picture.getBasicPixel(e.getX(), e.getY()-1);
				int up2 = picture.getBasicPixel(e.getX(), e.getY()-2);
			
				int left1 = picture.getBasicPixel(e.getX()-1, e.getY());
				int left2 = picture.getBasicPixel(e.getX()-2, e.getY());

				int right1 = picture.getBasicPixel(e.getX()+1, e.getY());
				int right2 = picture.getBasicPixel(e.getX()+2, e.getY());
			
				int rightUpDia1 = picture.getBasicPixel(e.getX()+1, e.getY()-1);
				int rightUpDia2 = picture.getBasicPixel(e.getX()+2, e.getY()-2);
			
				int rightDownDia1 = picture.getBasicPixel(e.getX()+1, e.getY()+1);
				int rightDownDia2 = picture.getBasicPixel(e.getX()+2, e.getY()+2);
			
				int leftUpDia1 = picture.getBasicPixel(e.getX()-1, e.getY()-1);
				int leftUpDia2 = picture.getBasicPixel(e.getX()-2, e.getY()-2);
			
				int leftDownDia1 = picture.getBasicPixel(e.getX()-1, e.getY()+1);
				int leftDownDia2 = picture.getBasicPixel(e.getX()-2, e.getY()+2);
			
				//top outer
				int outer1 = picture.getBasicPixel(e.getX()-1, e.getY()-2);
				int outer2 = picture.getBasicPixel(e.getX()+1, e.getY()-2);
				//left outer
				int outer3 = picture.getBasicPixel(e.getX()+2, e.getY()-1);
				int outer4 = picture.getBasicPixel(e.getX()+2, e.getY()+1);
				//bottom outer
				int outer5 = picture.getBasicPixel(e.getX()+1, e.getY()+2);
				int outer6 = picture.getBasicPixel(e.getX()-1, e.getY()+2);
				//right outer
				int outer7 = picture.getBasicPixel(e.getX()-2, e.getY()+1);
				int outer8 = picture.getBasicPixel(e.getX()-2, e.getY()-1);
			

				//makes the pixels larger by redrawing them in squares
				//Center Pixel
				createZoom(50,30,center);
			
				//up 1 pixel
				createZoom(50,20,up1);
				//up 2 pixels
				createZoom(50,10,up2);			
			
				//down 1 pixel
				createZoom(50,40,down1);
				//down 2 pixels
				createZoom(50,50,down2);
			
				//right 1 pixel
				createZoom(60,30,right1);
				//right 2 pixels
				createZoom(70,30,right2);
			
				//left 1 pixel
				createZoom(40,30,left1);
				//left 2 pixels
				createZoom(30,30,left2);
			
				//right diagonal up 1 pixel
				createZoom(60,20,rightUpDia1);
				//right diagonal up 2 pixels
				createZoom(70,10,rightUpDia2);
			
				//right diagonal down 1 pixel
				createZoom(60,40,rightDownDia1);
				//right diagonal down 2 pixels
				createZoom(70,50,rightDownDia2);
			
				//left diagonal up 1 pixel
				createZoom(40,20,leftUpDia1);
				//left diagonal up 2 pixels
				createZoom(30,10,leftUpDia2);
			
				//right diagonal down 1 pixel
				createZoom(40,40,leftDownDia1);
				//right diagonal down 2 pixels
				createZoom(30,50,leftDownDia2);		

				//top gaps
				createZoom(40,10,outer1);
				createZoom(60,10,outer2);
			
				//right gaps
				createZoom(70,20,outer3);
				createZoom(70,40,outer4);
			
				//bottom gaps
				createZoom(60,50,outer5);
				createZoom(40,50,outer6);
			
				//left gaps
				createZoom(30,40,outer7);
				createZoom(30,20,outer8);
			}
			else
			{
				//makes blank
				for(int x=0;x<=100;x++)
				{
					for(int y=0;y<=60;y++)
					{
						zoomPic.setBasicPixel(x,y,0);
					}
				}
			}
				
			//repaints the zoom window
			zoomPanel.repaint();
			
		}
		catch(ArrayIndexOutOfBoundsException exception)
		{
			xValue.setText("X: N/A");
			yValue.setText("Y: N/A");
			rValue.setText("R: N/A");
			gValue.setText("G: N/A");
			bValue.setText("B: N/A");
			//makes zoom window blank
			for(int x=0;x<=100;x++)
			{
				for(int y=0;y<=60;y++)
				{
					zoomPic.setBasicPixel(x,y,0);
				}
			}
			zoomPanel.repaint();
		}
		catch(NullPointerException exception)
		{
		}
		
	}
	
	public void mouseDragged(MouseEvent e)
	{
	}
	
	//****************************************//
	//              Test Main                 //
	//****************************************//
	
	/**
     * Test Main
     */
	public static void main( String args[])
    {
		JavaPicture p = new JavaPicture();
		p.loadImage("/net/hu11/awilson/Desktop/Help.jpg");
		
		PictureViewer test;
		test= new PictureViewer(p);
    }
}
